@@ -18,6 +18,7 @@ Archive::Entry::Entry(QObject *parent, const QString &fullPath, const QString &r
     , m_parent(qobject_cast<Entry*>(parent))
     , m_size(0)
     , m_compressedSize(0)
+    , m_sparseSize(0)
     , m_isDirectory(false)
     , m_isExecutable(false)
     , m_isPasswordProtected(false)
@@ -30,6 +31,16 @@ Archive::Entry::~Entry()
 {
 }
 
+qulonglong Archive::Entry::size() const
+{
+    return m_size;
+}
+
+qulonglong Archive::Entry::sparseSize() const
+{
+    return m_sparseSize;
+}
+
 void Archive::Entry::copyMetaData(const Archive::Entry *sourceEntry)
 {
     setProperty("fullPath", sourceEntry->property("fullPath"));
@@ -38,6 +49,7 @@ void Archive::Entry::copyMetaData(const Archive::Entry *sourceEntry)
     setProperty("group", sourceEntry->property("group"));
     setProperty("size", sourceEntry->property("size"));
     setProperty("compressedSize", sourceEntry->property("compressedSize"));
+    setProperty("sparseSize", sourceEntry->property("sparseSize"));
     setProperty("link", sourceEntry->property("link"));
     setProperty("ratio", sourceEntry->property("ratio"));
     setProperty("CRC", sourceEntry->property("CRC"));
@@ -107,6 +119,15 @@ QString Archive::Entry::fullPath(PathFormat format) const
     }
 }
 
+QString Archive::Entry::displayName() const
+{
+    if (m_displayName.isEmpty()) {
+        return m_name;
+    }
+
+    return m_displayName;
+}
+
 QString Archive::Entry::name() const
 {
     return m_name;
@@ -117,6 +138,11 @@ QStringView Archive::Entry::nameView() const
     return m_name;
 }
 
+void Archive::Entry::setDisplayName(const QString &displayName)
+{
+    m_displayName = displayName;
+}
+
 void Archive::Entry::setIsDirectory(const bool isDirectory)
 {
     m_isDirectory = isDirectory;
@@ -197,7 +223,7 @@ QIcon Archive::Entry::icon() const
             static QIcon directoryIcon = QIcon::fromTheme(db.mimeTypeForName(QStringLiteral("inode/directory")).iconName());
             m_icon = directoryIcon;
         } else {
-            m_icon = QIcon::fromTheme(db.mimeTypeForFile(m_name, QMimeDatabase::MatchMode::MatchExtension).iconName());
+            m_icon = QIcon::fromTheme(db.mimeTypeForFile(displayName(), QMimeDatabase::MatchMode::MatchExtension).iconName());
         }
     }
 
